use strict;
use warnings;
use Test::More;

use_ok 'Data::Object::Role::Hash';

can_ok 'Data::Object::Role::Hash', 'clear';
can_ok 'Data::Object::Role::Hash', 'count';
can_ok 'Data::Object::Role::Hash', 'data';
can_ok 'Data::Object::Role::Hash', 'defined';
can_ok 'Data::Object::Role::Hash', 'delete';
can_ok 'Data::Object::Role::Hash', 'detract';
can_ok 'Data::Object::Role::Hash', 'dump';
can_ok 'Data::Object::Role::Hash', 'each';
can_ok 'Data::Object::Role::Hash', 'each_key';
can_ok 'Data::Object::Role::Hash', 'each_n_values';
can_ok 'Data::Object::Role::Hash', 'each_value';
can_ok 'Data::Object::Role::Hash', 'empty';
can_ok 'Data::Object::Role::Hash', 'eq';
can_ok 'Data::Object::Role::Hash', 'exists';
can_ok 'Data::Object::Role::Hash', 'filter_exclude';
can_ok 'Data::Object::Role::Hash', 'filter_include';
can_ok 'Data::Object::Role::Hash', 'fold';
can_ok 'Data::Object::Role::Hash', 'ge';
can_ok 'Data::Object::Role::Hash', 'get';
can_ok 'Data::Object::Role::Hash', 'grep';
can_ok 'Data::Object::Role::Hash', 'gt';
can_ok 'Data::Object::Role::Hash', 'head';
can_ok 'Data::Object::Role::Hash', 'invert';
can_ok 'Data::Object::Role::Hash', 'iterator';
can_ok 'Data::Object::Role::Hash', 'join';
can_ok 'Data::Object::Role::Hash', 'keys';
can_ok 'Data::Object::Role::Hash', 'le';
can_ok 'Data::Object::Role::Hash', 'length';
can_ok 'Data::Object::Role::Hash', 'list';
can_ok 'Data::Object::Role::Hash', 'lookup';
can_ok 'Data::Object::Role::Hash', 'lt';
can_ok 'Data::Object::Role::Hash', 'map';
can_ok 'Data::Object::Role::Hash', 'merge';
can_ok 'Data::Object::Role::Hash', 'methods';
can_ok 'Data::Object::Role::Hash', 'ne';
can_ok 'Data::Object::Role::Hash', 'pairs';
can_ok 'Data::Object::Role::Hash', 'print';
can_ok 'Data::Object::Role::Hash', 'reset';
can_ok 'Data::Object::Role::Hash', 'reverse';
can_ok 'Data::Object::Role::Hash', 'roles';
can_ok 'Data::Object::Role::Hash', 'say';
can_ok 'Data::Object::Role::Hash', 'set';
can_ok 'Data::Object::Role::Hash', 'slice';
can_ok 'Data::Object::Role::Hash', 'sort';
can_ok 'Data::Object::Role::Hash', 'tail';
can_ok 'Data::Object::Role::Hash', 'throw';
can_ok 'Data::Object::Role::Hash', 'type';
can_ok 'Data::Object::Role::Hash', 'unfold';
can_ok 'Data::Object::Role::Hash', 'values';

ok 1 and done_testing;
